@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: libspeechd.c,v 1.37 2008-12-23 09:15:32 pdm Exp $
  */
@@ -82,8 +80,20 @@ const int range_high = 100;
 
 pthread_mutex_t spd_logging_mutex;
 
-#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
-/* Added by Willie Walker - strndup and getline are gcc-isms */
+struct SPDConnection_threaddata {
+	pthread_t events_thread;
+	pthread_cond_t cond_reply_ready;
+	pthread_mutex_t mutex_reply_ready;
+	pthread_cond_t cond_reply_ack;
+	pthread_mutex_t mutex_reply_ack;
+};
+
+/*
+ * Added by Willie Walker - strndup and getline were GNU libc extensions
+ * that were adopted in the POSIX.1-2008 standard, but are not yet found
+ * on all systems.
+ */
+#ifndef HAVE_STRNDUP
 char *strndup(const char *s, size_t n)
 {
 	size_t nAvail;
@@ -102,11 +112,13 @@ char *strndup(const char *s, size_t n)
 
 	return p;
 }
+#endif /* HAVE_STRNDUP */
 
+#ifndef HAVE_GETLINE
 #define BUFFER_LEN 256
 ssize_t getline(char **lineptr, size_t * n, FILE * f)
 {
-	char ch;
+	int ch;
 	size_t m = 0;
 	ssize_t buf_len = 0;
 	char *buf = NULL;
@@ -143,7 +155,7 @@ ssize_t getline(char **lineptr, size_t * n, FILE * f)
 		return m;
 	}
 }
-#endif /* !(defined(__GLIBC__) && defined(_GNU_SOURCE)) */
+#endif /* HAVE_GETLINE */
 
 /* --------------------- Public functions ------------------------- */
 
@@ -321,7 +333,7 @@ static int
 spawn_server(SPDConnectionAddress * address, int is_localhost,
 	     gchar ** spawn_error)
 {
-	gchar **speechd_cmd = malloc(16 * sizeof(char *));
+	gchar *speechd_cmd[16];
 	gchar *stderr_output;
 	gboolean spawn_ok;
 	GError *gerror = NULL;
@@ -384,11 +396,11 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 			 SPDConnectionAddress * address, int autospawn,
 			 char **error_result)
 {
-	SPDConnection *connection;
+	SPDConnection *connection = NULL;
 	SPDConnectionAddress *defaultAddress = NULL;
-	char *set_client_name;
-	char *conn_name;
-	char *usr_name;
+	char *set_client_name = NULL;
+	char *conn_name = NULL;
+	char *usr_name = NULL;
 	int ret;
 	char tcp_no_delay = 1;
 
@@ -402,7 +414,6 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	struct sockaddr_un address_unix;
 	struct sockaddr *sock_address;
 	size_t sock_address_len;
-	gchar *resolve_error;
 
 	_init_debug();
 
@@ -430,23 +441,26 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 			assert(err);
 			*error_result = err;
 			SPD_DBG(*error_result);
-			return NULL;
+			goto out;
 		}
 	}
 
 	/* Connect to server using the selected method */
 	connection = malloc(sizeof(SPDConnection));
 	if (address->method == SPD_METHOD_INET_SOCKET) {
+		gchar *resolve_error;
 		host_ip =
 		    resolve_host(address->inet_socket_host, &is_localhost,
 				 &resolve_error);
 		if (host_ip == NULL) {
 			*error_result = strdup(resolve_error);
 			g_free(resolve_error);
-			SPDConnectionAddress__free(defaultAddress);
-			return NULL;
+			free(connection);
+			connection = NULL;
+			goto out;
 		}
 		address_inet.sin_addr.s_addr = inet_addr(host_ip);
+		free(host_ip);
 		address_inet.sin_port = htons(address->inet_socket_port);
 		address_inet.sin_family = AF_INET;
 		connection->socket = socket(AF_INET, SOCK_STREAM, 0);
@@ -464,6 +478,11 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	} else
 		SPD_FATAL("Unsupported connection method for spd_open2()");
 
+	if (connection->socket < 0) {
+		free(connection);
+		connection = NULL;
+		goto out;
+	}
 	ret = connect(connection->socket, sock_address, sock_address_len);
 	if (ret == -1) {
 		/* Suppose server might not be running, try to autospawn (autostart) it */
@@ -497,8 +516,9 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 				assert(0);
 			SPD_DBG(*error_result);
 			close(connection->socket);
-			SPDConnectionAddress__free(defaultAddress);
-			return NULL;
+			free(connection);
+			connection = NULL;
+			goto out;
 		}
 		g_free(spawn_report);
 	}
@@ -525,29 +545,27 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	if (ret)
 		SPD_FATAL("Can't set buffering, setvbuf failed.");
 
-	connection->ssip_mutex = malloc(sizeof(pthread_mutex_t));
-	pthread_mutex_init(connection->ssip_mutex, NULL);
+	pthread_mutex_init(&connection->ssip_mutex, NULL);
 
 	if (mode == SPD_MODE_THREADED) {
 		SPD_DBG
 		    ("Initializing threads, condition variables and mutexes...");
-		connection->events_thread = malloc(sizeof(pthread_t));
-		connection->cond_reply_ready = malloc(sizeof(pthread_cond_t));
-		connection->mutex_reply_ready = malloc(sizeof(pthread_mutex_t));
-		connection->cond_reply_ack = malloc(sizeof(pthread_cond_t));
-		connection->mutex_reply_ack = malloc(sizeof(pthread_mutex_t));
-		pthread_cond_init(connection->cond_reply_ready, NULL);
-		pthread_mutex_init(connection->mutex_reply_ready, NULL);
-		pthread_cond_init(connection->cond_reply_ack, NULL);
-		pthread_mutex_init(connection->mutex_reply_ack, NULL);
+		connection->td = malloc(sizeof(*connection->td));
+		pthread_cond_init(&connection->td->cond_reply_ready, NULL);
+		pthread_mutex_init(&connection->td->mutex_reply_ready, NULL);
+		pthread_cond_init(&connection->td->cond_reply_ack, NULL);
+		pthread_mutex_init(&connection->td->mutex_reply_ack, NULL);
 		ret =
-		    pthread_create(connection->events_thread, NULL,
+		    pthread_create(&connection->td->events_thread, NULL,
 				   spd_events_handler, connection);
 		if (ret != 0) {
 			*error_result = strdup("Thread initialization failed");
 			SPD_DBG(*error_result);
-			SPDConnectionAddress__free(defaultAddress);
-			return NULL;
+			fclose(connection->stream);
+			close(connection->socket);
+			free(connection);
+			connection = NULL;
+			goto out;
 		}
 	}
 
@@ -556,6 +574,8 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	    g_strdup_printf("SET SELF CLIENT_NAME \"%s:%s:%s\"", usr_name,
 			    client_name, conn_name);
 	ret = spd_execute_command_wo_mutex(connection, set_client_name);
+
+out:
 	free(usr_name);
 	free(conn_name);
 	free(set_client_name);
@@ -565,7 +585,7 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 
 #define RET(r) \
 	{ \
-		pthread_mutex_unlock(connection->ssip_mutex); \
+		pthread_mutex_unlock(&connection->ssip_mutex); \
 		return r; \
 	}
 
@@ -573,29 +593,25 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 void spd_close(SPDConnection * connection)
 {
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	if (connection->mode == SPD_MODE_THREADED) {
-		pthread_cancel(*connection->events_thread);
-		pthread_mutex_destroy(connection->mutex_reply_ready);
-		pthread_mutex_destroy(connection->mutex_reply_ack);
-		pthread_cond_destroy(connection->cond_reply_ready);
-		pthread_cond_destroy(connection->cond_reply_ack);
-		pthread_join(*connection->events_thread, NULL);
+		pthread_cancel(connection->td->events_thread);
+		pthread_mutex_destroy(&connection->td->mutex_reply_ready);
+		pthread_mutex_destroy(&connection->td->mutex_reply_ack);
+		pthread_cond_destroy(&connection->td->cond_reply_ready);
+		pthread_cond_destroy(&connection->td->cond_reply_ack);
+		pthread_join(connection->td->events_thread, NULL);
 		connection->mode = SPD_MODE_SINGLE;
-		free(connection->mutex_reply_ready);
-		free(connection->mutex_reply_ack);
-		free(connection->cond_reply_ready);
-		free(connection->cond_reply_ack);
+		free(connection->td);
 	}
 
 	/* close the socket */
 	close(connection->socket);
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
-	pthread_mutex_destroy(connection->ssip_mutex);
-	free(connection->ssip_mutex);
+	pthread_mutex_destroy(&connection->ssip_mutex);
 	free(connection);
 }
 
@@ -674,7 +690,7 @@ int spd_say(SPDConnection * connection, SPDPriority priority, const char *text)
 	int prepare_failed = 0;
 
 	if (text != NULL) {
-		pthread_mutex_lock(connection->ssip_mutex);
+		pthread_mutex_lock(&connection->ssip_mutex);
 
 		prepare_failed =
 		    spd_say_prepare(connection, priority, text, &escaped_text);
@@ -682,7 +698,7 @@ int spd_say(SPDConnection * connection, SPDPriority priority, const char *text)
 			msg_id = spd_say_sending(connection, escaped_text);
 
 		free(escaped_text);
-		pthread_mutex_unlock(connection->ssip_mutex);
+		pthread_mutex_unlock(&connection->ssip_mutex);
 	} else {
 		SPD_DBG("spd_say called with a NULL argument for <text>");
 	}
@@ -796,7 +812,7 @@ spd_key(SPDConnection * connection, SPDPriority priority, const char *key_name)
 	if (key_name == NULL)
 		return -1;
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	ret = spd_set_priority(connection, priority);
 	if (ret)
@@ -808,7 +824,7 @@ spd_key(SPDConnection * connection, SPDPriority priority, const char *key_name)
 	if (ret)
 		RET(-1);
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
 	return 0;
 }
@@ -825,7 +841,7 @@ spd_char(SPDConnection * connection, SPDPriority priority,
 	if (strlen(character) > 6)
 		return -1;
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	ret = spd_set_priority(connection, priority);
 	if (ret)
@@ -836,7 +852,7 @@ spd_char(SPDConnection * connection, SPDPriority priority,
 	if (ret)
 		RET(-1);
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
 	return 0;
 }
@@ -848,7 +864,7 @@ spd_wchar(SPDConnection * connection, SPDPriority priority, wchar_t wcharacter)
 	char character[8];
 	int ret;
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	ret = wcrtomb(character, wcharacter, NULL);
 	if (ret <= 0)
@@ -864,7 +880,7 @@ spd_wchar(SPDConnection * connection, SPDPriority priority, wchar_t wcharacter)
 	if (ret)
 		RET(-1);
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
 	return 0;
 }
@@ -879,7 +895,7 @@ spd_sound_icon(SPDConnection * connection, SPDPriority priority,
 	if (icon_name == NULL)
 		return -1;
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	ret = spd_set_priority(connection, priority);
 	if (ret)
@@ -891,14 +907,14 @@ spd_sound_icon(SPDConnection * connection, SPDPriority priority,
 	if (ret)
 		RET(-1);
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
 	return 0;
 }
 
-int
-spd_w_set_punctuation(SPDConnection * connection, SPDPunctuation type,
-		      const char *who)
+// Set functions for Punctuation
+int spd_w_set_punctuation(SPDConnection * connection, SPDPunctuation type,
+			  const char *who)
 {
 	char command[32];
 	int ret;
@@ -915,9 +931,27 @@ spd_w_set_punctuation(SPDConnection * connection, SPDPunctuation type,
 	return ret;
 }
 
-int
-spd_w_set_capital_letters(SPDConnection * connection, SPDCapitalLetters type,
-			  const char *who)
+int spd_set_punctuation(SPDConnection * connection, SPDPunctuation type)
+{
+	return spd_w_set_punctuation(connection, type, SPD_SELF);
+}
+
+int spd_set_punctuation_all(SPDConnection * connection, SPDPunctuation type)
+{
+	return spd_w_set_punctuation(connection, type, SPD_ALLCLIENTS);
+}
+
+int spd_set_punctuation_uid(SPDConnection * connection, SPDPunctuation type,
+			    unsigned int uid)
+{
+	char who[8];
+	sprintf(who, "%d", uid);
+	return spd_w_set_punctuation(connection, type, who);
+}
+
+// Set functions for Capital Letters
+int spd_w_set_capital_letters(SPDConnection * connection,
+			      SPDCapitalLetters type, const char *who)
 {
 	char command[64];
 	int ret;
@@ -934,9 +968,28 @@ spd_w_set_capital_letters(SPDConnection * connection, SPDCapitalLetters type,
 	return ret;
 }
 
-int
-spd_w_set_spelling(SPDConnection * connection, SPDSpelling type,
-		   const char *who)
+int spd_set_capital_letters(SPDConnection * connection, SPDCapitalLetters type)
+{
+	return spd_w_set_capital_letters(connection, type, SPD_SELF);
+}
+
+int spd_set_capital_letters_all(SPDConnection * connection,
+				SPDCapitalLetters type)
+{
+	return spd_w_set_capital_letters(connection, type, SPD_ALLCLIENTS);
+}
+
+int spd_set_capital_letters_uid(SPDConnection * connection,
+				SPDCapitalLetters type, unsigned int uid)
+{
+	char who[8];
+	sprintf(who, "%d", uid);
+	return spd_w_set_capital_letters(connection, type, who);
+}
+
+// Set functions for Spelling
+int spd_w_set_spelling(SPDConnection * connection, SPDSpelling type,
+		       const char *who)
 {
 	char command[32];
 	int ret;
@@ -951,6 +1004,24 @@ spd_w_set_spelling(SPDConnection * connection, SPDSpelling type,
 	return ret;
 }
 
+int spd_set_spelling(SPDConnection * connection, SPDSpelling type)
+{
+	return spd_w_set_spelling(connection, type, SPD_SELF);
+}
+
+int spd_set_spelling_all(SPDConnection * connection, SPDSpelling type)
+{
+	return spd_w_set_spelling(connection, type, SPD_ALLCLIENTS);
+}
+
+int spd_set_spelling_uid(SPDConnection * connection, SPDSpelling type,
+			 unsigned int uid)
+{
+	char who[8];
+	sprintf(who, "%d", uid);
+	return spd_w_set_spelling(connection, type, who);
+}
+
 int spd_set_data_mode(SPDConnection * connection, SPDDataMode mode)
 {
 	char command[32];
@@ -966,9 +1037,9 @@ int spd_set_data_mode(SPDConnection * connection, SPDDataMode mode)
 	return ret;
 }
 
-int
-spd_w_set_voice_type(SPDConnection * connection, SPDVoiceType type,
-		     const char *who)
+// Set functions for Voice type
+int spd_w_set_voice_type(SPDConnection * connection, SPDVoiceType type,
+			 const char *who)
 {
 	static char command[64];
 
@@ -1004,6 +1075,39 @@ spd_w_set_voice_type(SPDConnection * connection, SPDVoiceType type,
 	return spd_execute_command(connection, command);
 }
 
+int spd_set_voice_type(SPDConnection * connection, SPDVoiceType type)
+{
+	return spd_w_set_voice_type(connection, type, SPD_SELF);
+}
+
+int spd_set_voice_type_all(SPDConnection * connection, SPDVoiceType type)
+{
+	return spd_w_set_voice_type(connection, type, SPD_ALLCLIENTS);
+}
+
+int spd_set_voice_type_uid(SPDConnection * connection, SPDVoiceType type,
+			   unsigned int uid)
+{
+	char who[8];
+	sprintf(who, "%d", uid);
+	return spd_w_set_voice_type(connection, type, who);
+}
+
+// Set function for Voice type
+SPDVoiceType spd_get_voice_type(SPDConnection * connection)
+{
+	char *command;
+	SPDVoiceType ret;
+	int err;
+	char *reply = NULL;
+	command = g_strdup_printf("GET voice_type");
+	spd_execute_command_with_reply(connection, command, &reply);
+	free(command);
+	ret = get_param_int(reply, 1, &err);
+	free(reply);
+	return ret;
+}
+
 static int spd_w_set_command_int(SPDConnection * connection,
 				 const char *ssip_name, signed int val,
 				 const char *who)
@@ -1062,42 +1166,7 @@ static char *spd_get_command_str(SPDConnection * connection,
 	return ret;
 }
 
-#define SPD_SET_COMMAND_SPECIAL(param, type) \
-	int \
-	spd_set_ ## param (SPDConnection *connection, type val) \
-	{ \
-		return spd_w_set_ ## param (connection, val, SPD_SELF); \
-	} \
-	int \
-	spd_set_ ## param ## _all(SPDConnection *connection, type val) \
-	{ \
-		return spd_w_set_ ## param (connection, val, SPD_ALLCLIENTS); \
-	} \
-	int \
-	spd_set_ ## param ## _uid(SPDConnection *connection, type val, unsigned int uid) \
-	{ \
-		char who[8]; \
-		sprintf(who, "%d", uid); \
-		return spd_w_set_ ## param (connection, val, who); \
-	}
-
-#define SPD_GET_COMMAND_SPECIAL(param, type) \
-	type \
-	spd_get_ ## param (SPDConnection *connection) \
-	{ \
-		char *command; \
-		type ret; \
-		int err; \
-		char *reply = NULL; \
-		command = g_strdup_printf("GET " #param); \
-		spd_execute_command_with_reply(connection, command, &reply); \
-		free(command); \
-		ret = get_param_int(reply, 1, &err); \
-		free(reply); \
-		return ret; \
-	}
-
-// Set methods for rate
+// Set functions for rate
 int spd_set_voice_rate(SPDConnection * connection, signed int rate)
 {
 	return spd_w_set_command_int(connection, SPD_RATE, rate, SPD_SELF);
@@ -1117,13 +1186,13 @@ int spd_set_voice_rate_uid(SPDConnection * connection, signed int rate,
 	return spd_w_set_command_int(connection, SPD_RATE, rate, who);
 }
 
-// Get method for rate
+// Get function for rate
 int spd_get_voice_rate(SPDConnection * connection)
 {
 	return spd_get_command_int(connection, SPD_RATE);
 }
 
-// Set methods for pitch
+// Set functions for pitch
 int spd_set_voice_pitch(SPDConnection * connection, signed int pitch)
 {
 	return spd_w_set_command_int(connection, SPD_PITCH, pitch, SPD_SELF);
@@ -1143,13 +1212,36 @@ int spd_set_voice_pitch_uid(SPDConnection * connection, signed int pitch,
 	return spd_w_set_command_int(connection, SPD_PITCH, pitch, who);
 }
 
-// Get method for pitch
+// Get function for pitch
 int spd_get_voice_pitch(SPDConnection * connection)
 {
 	return spd_get_command_int(connection, SPD_PITCH);
 }
 
-// Set methods for volume
+// Set functions for pitch_range
+int spd_set_voice_pitch_range(SPDConnection * connection,
+			      signed int pitch_range)
+{
+	return spd_w_set_command_int(connection, SPD_PITCH_RANGE, pitch_range,
+				     SPD_SELF);
+}
+
+int spd_set_voice_pitch_range_all(SPDConnection * connection,
+				  signed int pitch_range)
+{
+	return spd_w_set_command_int(connection, SPD_PITCH, pitch_range,
+				     SPD_ALLCLIENTS);
+}
+
+int spd_set_voice_pitch_range_uid(SPDConnection * connection,
+				  signed int pitch_range, unsigned int uid)
+{
+	char who[8];
+	sprintf(who, "%d", uid);
+	return spd_w_set_command_int(connection, SPD_PITCH, pitch_range, who);
+}
+
+// Set functions for volume
 int spd_set_volume(SPDConnection * connection, signed int volume)
 {
 	return spd_w_set_command_int(connection, SPD_VOLUME, volume, SPD_SELF);
@@ -1169,13 +1261,13 @@ int spd_set_volume_uid(SPDConnection * connection, signed int volume,
 	return spd_w_set_command_int(connection, SPD_VOLUME, volume, who);
 }
 
-// Get method for volume
+// Get function for volume
 int spd_get_volume(SPDConnection * connection)
 {
 	return spd_get_command_int(connection, SPD_VOLUME);
 }
 
-// Set methods for language
+// Set functions for language
 int spd_set_language(SPDConnection * connection, const char *language)
 {
 	return spd_w_set_command_str(connection, SPD_LANGUAGE, language,
@@ -1196,13 +1288,13 @@ int spd_set_language_uid(SPDConnection * connection, const char *language,
 	return spd_w_set_command_str(connection, SPD_LANGUAGE, language, who);
 }
 
-// Get method for language
+// Get function for language
 char *spd_get_language(SPDConnection * connection)
 {
 	return spd_get_command_str(connection, SPD_LANGUAGE);
 }
 
-// Set methods for output_module
+// Set functions for output_module
 int spd_set_output_module(SPDConnection * connection, const char *output_module)
 {
 	return spd_w_set_command_str(connection, SPD_OUTPUT_MODULE,
@@ -1225,13 +1317,13 @@ int spd_set_output_module_uid(SPDConnection * connection,
 				     output_module, who);
 }
 
-// Get method for output_module
+// Get function for output_module
 char *spd_get_output_module(SPDConnection * connection)
 {
 	return spd_get_command_str(connection, SPD_OUTPUT_MODULE);
 }
 
-// Set methods for synthesis_voice
+// Set functions for synthesis_voice
 int spd_set_synthesis_voice(SPDConnection * connection, const char *voice_name)
 {
 	return spd_w_set_command_str(connection, SPD_SYNTHESIS_VOICE,
@@ -1254,12 +1346,6 @@ int spd_set_synthesis_voice_uid(SPDConnection * connection,
 				     voice_name, who);
 }
 
-SPD_SET_COMMAND_SPECIAL(punctuation, SPDPunctuation)
-    SPD_SET_COMMAND_SPECIAL(capital_letters, SPDCapitalLetters)
-    SPD_SET_COMMAND_SPECIAL(spelling, SPDSpelling)
-    SPD_SET_COMMAND_SPECIAL(voice_type, SPDVoiceType)
-    SPD_GET_COMMAND_SPECIAL(voice_type, SPDVoiceType)
-#undef SPD_SET_COMMAND_SPECIAL
 int
 spd_set_notification_on(SPDConnection * connection,
 			SPDNotification notification)
@@ -1306,7 +1392,7 @@ spd_set_notification(SPDConnection * connection, SPDNotification notification,
 		return -1;
 	}
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	NOTIFICATION_SET(SPD_INDEX_MARKS, "index_marks");
 	NOTIFICATION_SET(SPD_BEGIN, "begin");
@@ -1316,7 +1402,7 @@ spd_set_notification(SPDConnection * connection, SPDNotification notification,
 	NOTIFICATION_SET(SPD_RESUME, "resume");
 	NOTIFICATION_SET(SPD_ALL, "all");
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
 	return 0;
 }
@@ -1373,12 +1459,12 @@ SPDVoice **spd_list_synthesis_voices(SPDConnection * connection)
 	svoices = (SPDVoice **) malloc((num_items + 1) * sizeof(SPDVoice *));
 
 	for (i = 0; i <= num_items; i++) {
-		const char delimiters[] = " ";
+		const char delimiters[] = "\t";
 		char *running;
 
 		if (svoices_str[i] == NULL)
 			break;
-		running = strdup(svoices_str[i]);
+		running = svoices_str[i];
 
 		svoices[i] = (SPDVoice *) malloc(sizeof(SPDVoice));
 		svoices[i]->name = strsep(&running, delimiters);
@@ -1386,6 +1472,7 @@ SPDVoice **spd_list_synthesis_voices(SPDConnection * connection)
 		svoices[i]->variant = strsep(&running, delimiters);
 		assert(svoices[i]->name != NULL);
 	}
+	free(svoices_str);
 
 	svoices[num_items] = NULL;
 
@@ -1481,7 +1568,7 @@ int spd_execute_command(SPDConnection * connection, char *command)
 	char *reply;
 	int ret;
 
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	ret = spd_execute_command_with_reply(connection, command, &reply);
 	if (ret) {
@@ -1489,7 +1576,7 @@ int spd_execute_command(SPDConnection * connection, char *command)
 	}
 	free(reply);
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 
 	return ret;
 }
@@ -1539,7 +1626,7 @@ spd_execute_command_with_reply(SPDConnection * connection, char *command,
 char *spd_send_data(SPDConnection * connection, const char *message, int wfr)
 {
 	char *reply;
-	pthread_mutex_lock(connection->ssip_mutex);
+	pthread_mutex_lock(&connection->ssip_mutex);
 
 	if (connection->stream == NULL)
 		RET(NULL);
@@ -1550,7 +1637,7 @@ char *spd_send_data(SPDConnection * connection, const char *message, int wfr)
 		RET(NULL);
 	}
 
-	pthread_mutex_unlock(connection->ssip_mutex);
+	pthread_mutex_unlock(&connection->ssip_mutex);
 	return reply;
 }
 
@@ -1569,13 +1656,14 @@ char *spd_send_data_wo_mutex(SPDConnection * connection, const char *message,
 	if (connection->mode == SPD_MODE_THREADED) {
 		/* Make sure we don't get the cond_reply_ready signal before we are in
 		   cond_wait() */
-		pthread_mutex_lock(connection->mutex_reply_ready);
+		pthread_mutex_lock(&connection->td->mutex_reply_ready);
 	}
 	/* write message to the socket */
 	SPD_DBG("Writing to socket");
 	if (!write(connection->socket, message, strlen(message))) {
 		SPD_DBG("Can't write to socket: %s", strerror(errno));
-		pthread_mutex_unlock(connection->mutex_reply_ready);
+		if (connection->mode == SPD_MODE_THREADED)
+			pthread_mutex_unlock(&connection->td->mutex_reply_ready);
 		return NULL;
 	}
 	SPD_DBG("Written to socket");
@@ -1587,30 +1675,31 @@ char *spd_send_data_wo_mutex(SPDConnection * connection, const char *message,
 			/* Wait until the reply is ready */
 			SPD_DBG
 			    ("Waiting for cond_reply_ready in spd_send_data_wo_mutex");
-			pthread_cond_wait(connection->cond_reply_ready,
-					  connection->mutex_reply_ready);
+			pthread_cond_wait(&connection->td->cond_reply_ready,
+					  &connection->td->mutex_reply_ready);
 			SPD_DBG("Condition for cond_reply_ready satisfied");
-			pthread_mutex_unlock(connection->mutex_reply_ready);
+			pthread_mutex_unlock(&connection->td->mutex_reply_ready);
 			SPD_DBG
 			    ("Reading the reply in spd_send_data_wo_mutex threaded mode");
 			/* Read the reply */
 			if (connection->reply != NULL) {
-				reply = strdup(connection->reply);
+				reply = connection->reply;
+				connection->reply = NULL;
 			} else {
 				SPD_DBG
 				    ("Error: Can't read reply, broken socket in spd_send_data.");
 				return NULL;
 			}
-			free(connection->reply);
 			bytes = strlen(reply);
 			if (bytes == 0) {
+				free(reply);
 				SPD_DBG("Error: Empty reply, broken socket.");
 				return NULL;
 			}
 			/* Signal the reply has been read */
-			pthread_mutex_lock(connection->mutex_reply_ack);
-			pthread_cond_signal(connection->cond_reply_ack);
-			pthread_mutex_unlock(connection->mutex_reply_ack);
+			pthread_mutex_lock(&connection->td->mutex_reply_ack);
+			pthread_cond_signal(&connection->td->cond_reply_ack);
+			pthread_mutex_unlock(&connection->td->mutex_reply_ack);
 		} else {
 			reply = get_reply(connection);
 		}
@@ -1618,7 +1707,7 @@ char *spd_send_data_wo_mutex(SPDConnection * connection, const char *message,
 			SPD_DBG("<< : |%s|\n", reply);
 	} else {
 		if (connection->mode == SPD_MODE_THREADED)
-			pthread_mutex_unlock(connection->mutex_reply_ready);
+			pthread_mutex_unlock(&connection->td->mutex_reply_ready);
 		SPD_DBG("<< : no reply expected");
 		return strdup("NO REPLY");
 	}
@@ -1790,36 +1879,35 @@ static void *spd_events_handler(void *conn)
 
 		} else {
 			/* This is a protocol reply */
-			pthread_mutex_lock(connection->mutex_reply_ready);
+			pthread_mutex_lock(&connection->td->mutex_reply_ready);
 			/* Prepare the reply to the reply buffer in connection */
 			if (reply != NULL) {
 				connection->reply = reply;
 			} else {
 				SPD_DBG("Connection reply is NULL");
 				connection->reply = NULL;
+				pthread_mutex_unlock(&connection->td->mutex_reply_ready);
 				break;
 			}
 			/* Signal the reply is available on the condition variable */
 			/* this order is correct and necessary */
-			pthread_cond_signal(connection->cond_reply_ready);
-			pthread_mutex_lock(connection->mutex_reply_ack);
-			pthread_mutex_unlock(connection->mutex_reply_ready);
+			pthread_cond_signal(&connection->td->cond_reply_ready);
+			pthread_mutex_lock(&connection->td->mutex_reply_ack);
+			pthread_mutex_unlock(&connection->td->mutex_reply_ready);
 			/* Wait until it has bean read */
-			pthread_cond_wait(connection->cond_reply_ack,
-					  connection->mutex_reply_ack);
-			pthread_mutex_unlock(connection->mutex_reply_ack);
+			pthread_cond_wait(&connection->td->cond_reply_ack,
+					  &connection->td->mutex_reply_ack);
+			pthread_mutex_unlock(&connection->td->mutex_reply_ack);
 			/* Continue */
 		}
 	}
 	/* In case of broken socket, we must still signal reply ready */
 	if (connection->reply == NULL) {
 		SPD_DBG("Signalling reply ready after communication failure");
-		pthread_mutex_unlock(connection->mutex_reply_ready);
-		pthread_mutex_unlock(connection->mutex_reply_ack);
 		if (connection->stream != NULL)
 			fclose(connection->stream);
 		connection->stream = NULL;
-		pthread_cond_signal(connection->cond_reply_ready);
+		pthread_cond_signal(&connection->td->cond_reply_ready);
 		pthread_exit(0);
 	}
 	return 0;		/* to please gcc */
@@ -1909,6 +1997,7 @@ static int get_param_int(char *reply, int num, int *err)
 	if (*tptr != '\0') {
 		/* this is not a number */
 		*err = -3;
+		free(rep_str);
 		return 0;
 	}
 	free(rep_str);
