@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
-- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: festival.c,v 1.82 2008-06-09 10:33:38 hanke Exp $
  */
@@ -256,7 +254,7 @@ int module_init(char **status_info)
 		if (ret == -1) {
 			ABORT
 			    ("Can't connect to Festival server. Check your configuration "
-			     "in etc/speechd-modules/festival.conf for the specified host and port "
+			     "in etc/speech-dispatcher/modules/festival.conf for the specified host and port "
 			     "and check if Festival is really running there, e.g. with telnet. "
 			     "Please see documentation for more info.");
 		} else if (ret == -2) {
@@ -393,7 +391,7 @@ int module_speak(char *data, size_t bytes, SPDMessageType msgtype)
 	g_free(*festival_message);
 	*festival_message = g_strdup(data);
 	if (*festival_message == NULL) {
-		DBG("Error: Copying data unsuccesful.");
+		DBG("Error: Copying data unsuccessful.");
 		return -1;
 	}
 
@@ -401,7 +399,7 @@ int module_speak(char *data, size_t bytes, SPDMessageType msgtype)
 	festival_speaking = 1;
 	sem_post(&festival_semaphore);
 
-	DBG("Festival: leaving write() normaly\n\r");
+	DBG("Festival: leaving write() normally\n\r");
 	return bytes;
 }
 
@@ -964,12 +962,10 @@ gint cache_counter_comp(gconstpointer a, gconstpointer b)
 void cache_debug_foreach_list_score(gpointer a, gpointer user)
 {
 	const TCounterEntry *A = a;
-	time_t t;
 
-	t = time(NULL);
 	DBG("key: %s      -> score %f (count: %d, dtime: %d)", A->key,
-	    ((float)A->count / (float)(t - A->start)), (int)A->count,
-	    (int)(t - A->start));
+	    ((float)A->count / (float)(time(NULL) - A->start)), (int)A->count,
+	    (int)(time(NULL) - A->start));
 }
 
 /* Remove 1/3 of the least used (according to cache_counter_comp) entries
