@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: generic.c,v 1.30 2008-07-30 09:15:51 hanke Exp $
  */
@@ -55,13 +53,16 @@ static char *execute_synth_str1;
 static char *execute_synth_str2;
 
 /* Internal functions prototypes */
+static void *get_ht_option(GHashTable * hash_table, const char *key);
 static void *_generic_speak(void *);
 static void _generic_child(TModuleDoublePipe dpipe, const size_t maxlen);
 static void generic_child_close(TModuleDoublePipe dpipe);
 
 void generic_set_rate(signed int rate);
 void generic_set_pitch(signed int pitch);
+void generic_set_pitch_range(signed int pitch_range);
 void generic_set_voice(SPDVoiceType voice);
+void generic_set_synthesis_voice(char *name);
 void generic_set_language(char *language);
 void generic_set_volume(signed int volume);
 void generic_set_punct(SPDPunctuation punct);
@@ -69,6 +70,8 @@ void generic_set_punct(SPDPunctuation punct);
 /* Fill the module_info structure with pointers to this modules functions */
 
 MOD_OPTION_1_STR(GenericExecuteSynth)
+    MOD_OPTION_1_STR(GenericCmdDependency)
+
     MOD_OPTION_1_INT(GenericMaxChunkLength)
     MOD_OPTION_1_STR(GenericDelimiters)
     MOD_OPTION_1_STR(GenericPunctNone)
@@ -83,12 +86,16 @@ MOD_OPTION_1_STR(GenericExecuteSynth)
     MOD_OPTION_1_INT(GenericPitchAdd)
     MOD_OPTION_1_FLOAT(GenericPitchMultiply)
     MOD_OPTION_1_INT(GenericPitchForceInteger)
+    MOD_OPTION_1_INT(GenericPitchRangeAdd)
+    MOD_OPTION_1_FLOAT(GenericPitchRangeMultiply)
+    MOD_OPTION_1_INT(GenericPitchRangeForceInteger)
     MOD_OPTION_1_INT(GenericVolumeAdd)
     MOD_OPTION_1_FLOAT(GenericVolumeMultiply)
     MOD_OPTION_1_INT(GenericVolumeForceInteger)
     MOD_OPTION_3_HT(GenericLanguage, code, name, charset)
 
 static char generic_msg_pitch_str[16];
+static char generic_msg_pitch_range_str[16];
 static char generic_msg_rate_str[16];
 static char generic_msg_volume_str[16];
 static char *generic_msg_voice_str = NULL;
@@ -102,6 +109,7 @@ int module_load(void)
 	INIT_SETTINGS_TABLES();
 
 	MOD_OPTION_1_STR_REG(GenericExecuteSynth, "");
+	MOD_OPTION_1_STR_REG(GenericCmdDependency, "");
 
 	REGISTER_DEBUG();
 
@@ -118,16 +126,21 @@ int module_load(void)
 	MOD_OPTION_1_FLOAT_REG(GenericPitchMultiply, 1);
 	MOD_OPTION_1_INT_REG(GenericPitchForceInteger, 0);
 
+	MOD_OPTION_1_INT_REG(GenericPitchRangeAdd, 0);
+	MOD_OPTION_1_FLOAT_REG(GenericPitchRangeMultiply, 1);
+	MOD_OPTION_1_INT_REG(GenericPitchRangeForceInteger, 0);
+
 	MOD_OPTION_1_INT_REG(GenericVolumeAdd, 0);
 	MOD_OPTION_1_FLOAT_REG(GenericVolumeMultiply, 1);
 	MOD_OPTION_1_INT_REG(GenericVolumeForceInteger, 0);
 
 	MOD_OPTION_HT_REG(GenericLanguage);
 
-	MOD_OPTION_1_STR_REG(GenericPunctNone, "--punct-none");
-	MOD_OPTION_1_STR_REG(GenericPunctSome, "--punct-some");
-	MOD_OPTION_1_STR_REG(GenericPunctAll, "--punct-all");
+	MOD_OPTION_1_STR_REG(GenericPunctNone, "");
+	MOD_OPTION_1_STR_REG(GenericPunctSome, "");
+	MOD_OPTION_1_STR_REG(GenericPunctAll, "");
 
+	module_register_available_voices();
 	module_register_settings_voices();
 
 	return 0;
@@ -142,6 +155,7 @@ int module_init(char **status_info)
 	DBG("GenericMaxChunkLength = %d\n", GenericMaxChunkLength);
 	DBG("GenericDelimiters = %s\n", GenericDelimiters);
 	DBG("GenericExecuteSynth = %s\n", GenericExecuteSynth);
+	DBG("GenericCmdDependency = %s\n", GenericCmdDependency);
 
 	generic_msg_language =
 	    (TGenericLanguage *) g_malloc(sizeof(TGenericLanguage));
@@ -171,7 +185,7 @@ int module_init(char **status_info)
 
 SPDVoice **module_list_voices(void)
 {
-	return NULL;
+	return module_list_registered_voices();
 }
 
 int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
@@ -184,11 +198,12 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 		DBG("Speaking when requested to write");
 		return 0;
 	}
-
+	UPDATE_STRING_PARAMETER(voice.name, generic_set_synthesis_voice);
 	UPDATE_STRING_PARAMETER(voice.language, generic_set_language);
 	UPDATE_PARAMETER(voice_type, generic_set_voice);
 	UPDATE_PARAMETER(punctuation_mode, generic_set_punct);
 	UPDATE_PARAMETER(pitch, generic_set_pitch);
+	UPDATE_PARAMETER(pitch_range, generic_set_pitch_range);
 	UPDATE_PARAMETER(rate, generic_set_rate);
 	UPDATE_PARAMETER(volume, generic_set_volume);
 
@@ -199,8 +214,8 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 		    generic_msg_language->charset);
 		tmp =
 		    (char *)g_convert_with_fallback(data, bytes,
-						    generic_msg_language->
-						    charset, "UTF-8",
+						    generic_msg_language->charset,
+						    "UTF-8",
 						    GenericRecodeFallback, NULL,
 						    NULL, NULL);
 	} else {
@@ -231,7 +246,7 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 	generic_speaking = 1;
 	sem_post(&generic_semaphore);
 
-	DBG("Generic: leaving write() normaly\n\r");
+	DBG("Generic: leaving write() normally\n\r");
 	return bytes;
 }
 
@@ -283,6 +298,18 @@ int module_close(void)
 
 /* Internal functions */
 
+static void *get_ht_option(GHashTable * hash_table, const char *key)
+{
+	void *option;
+	assert(key != NULL);
+
+	option = g_hash_table_lookup(hash_table, key);
+	if (option == NULL)
+		DBG("Requested option by key %s not found.\n", key);
+
+	return option;
+}
+
 /* Replace all occurances of 'token' in 'sting'
    with 'data' */
 char *string_replace(char *string, const char *token, const char *data)
@@ -358,8 +385,7 @@ void *_generic_speak(void *nothing)
 			generic_speaking = 0;
 			continue;
 
-		case 0:
-			{
+		case 0:{
 				char *e_string;
 				char *p;
 				char *tmpdir, *homedir;
@@ -406,6 +432,9 @@ void *_generic_speak(void *nothing)
 				e_string =
 				    string_replace(e_string, "$PITCH",
 						   generic_msg_pitch_str);
+				e_string =
+				    string_replace(e_string, "$PITCH_RANGE",
+						   generic_msg_pitch_range_str);
 				e_string =
 				    string_replace(e_string, "$RATE",
 						   generic_msg_rate_str);
@@ -568,6 +597,21 @@ void generic_set_pitch(int pitch)
 	}
 }
 
+void generic_set_pitch_range(int pitch_range)
+{
+	float hpitch_range;
+
+	hpitch_range =
+	    ((float)pitch_range) * GenericPitchRangeMultiply +
+	    GenericPitchRangeAdd;
+	if (!GenericPitchRangeForceInteger) {
+		snprintf(generic_msg_pitch_range_str, 15, "%.2f", hpitch_range);
+	} else {
+		snprintf(generic_msg_pitch_range_str, 15, "%d",
+			 (int)hpitch_range);
+	}
+}
+
 void generic_set_rate(int rate)
 {
 	float hrate;
@@ -599,7 +643,7 @@ void generic_set_language(char *lang)
 {
 
 	generic_msg_language =
-	    (TGenericLanguage *) module_get_ht_option(GenericLanguage, lang);
+	    (TGenericLanguage *) get_ht_option(GenericLanguage, lang);
 	if (generic_msg_language == NULL) {
 		DBG("Language %s not found in the configuration file, using defaults.", lang);
 		generic_msg_language =
@@ -624,6 +668,8 @@ void generic_set_language(char *lang)
 
 void generic_set_voice(SPDVoiceType voice)
 {
+	if (generic_msg_voice_str != NULL)
+		return;
 	assert(generic_msg_language);
 	generic_msg_voice_str =
 	    module_getvoice(generic_msg_language->code, voice);
@@ -632,6 +678,13 @@ void generic_set_voice(SPDVoiceType voice)
 	}
 }
 
+void generic_set_synthesis_voice(char *name)
+{
+	assert(msg_settings.voice.name);
+	if (module_existsvoice(msg_settings.voice.name))
+		generic_msg_voice_str = msg_settings.voice.name;
+}
+
 void generic_set_punct(SPDPunctuation punct)
 {
 	if (punct == SPD_PUNCT_NONE) {
