@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: module_utils.c,v 1.55 2008-07-10 15:37:18 hanke Exp $
  */
@@ -215,6 +213,10 @@ char *do_set(void)
 					      ((number >= -100)
 					       && (number <= 100)))
 				    else
+				SET_PARAM_NUM(pitch_range,
+					      ((number >= -100)
+					       && (number <= 100)))
+				    else
 				SET_PARAM_NUM(volume,
 					      ((number >= -100)
 					       && (number <= 100)))
@@ -320,6 +322,7 @@ char *do_audio(void)
 				    else
 				SET_AUDIO_STR(audio_pulse_min_length, 5)
 				    else
+				/* 6 reserved for speech-dispatcher module name */
 				err = 2;	/* Unknown parameter */
 		}
 		g_free(line);
@@ -429,11 +432,14 @@ char *do_debug(char *cmd_buf)
 		filename = cmd[2];
 		DBG("Additional logging into specific path %s requested",
 		    filename);
-		CustomDebugFile = fopen(filename, "w+");
-		if (CustomDebugFile == NULL) {
+		FILE *new_CustomDebugFile = fopen(filename, "w+");
+		if (new_CustomDebugFile == NULL) {
 			DBG("ERROR: Can't open custom debug file for logging: %d (%s)", errno, strerror(errno));
 			return g_strdup("303 CANT OPEN CUSTOM DEBUG FILE");
 		}
+		if (CustomDebugFile != NULL)
+			fclose(CustomDebugFile);
+		CustomDebugFile = new_CustomDebugFile;
 		if (Debug == 1)
 			Debug = 3;
 		else
@@ -484,7 +490,7 @@ char *do_list_voices(void)
 			variant = "none";
 		else
 			variant = voices[i]->variant;
-		g_string_append_printf(voice_list, "200-%s %s %s\n",
+		g_string_append_printf(voice_list, "200-%s\t%s\t%s\n",
 				       voices[i]->name, lang, variant);
 	}
 
@@ -512,10 +518,10 @@ void do_quit(void)
 	printf("210 OK QUIT\n");
 	fflush(stdout);
 
+	module_close();
+
 	spd_audio_close(module_audio_id);
 	module_audio_id = NULL;
-
-	module_close();
 	return;
 }
 
@@ -640,6 +646,12 @@ char *module_strip_ssml(char *message)
 		} else if (!strncmp(&(message[i]), "&amp;", 5)) {
 			i += 4;
 			out[n++] = '&';
+		} else if (!strncmp(&(message[i]), "&quot;", 6)) {
+			i += 5;
+			out[n++] = '"';
+		} else if (!strncmp(&(message[i]), "&apos;", 6)) {
+			i += 5;
+			out[n++] = '\'';
 		} else if (!omit || i == len)
 			out[n++] = message[i];
 	}
@@ -884,8 +896,7 @@ char *module_recode_to_iso(char *data, int bytes, char *language,
 
 	if (language == NULL)
 		recoded = g_strdup(data);
-
-	if (!strcmp(language, "cs"))
+	else if (!strcmp(language, "cs"))
 		recoded =
 		    (char *)g_convert_with_fallback(data, bytes, "ISO8859-2",
 						    "UTF-8", fallback, NULL,
@@ -948,7 +959,7 @@ void module_report_event_pause(void)
 
 /* --- CONFIGURATION --- */
 configoption_t *module_add_config_option(configoption_t * options,
-					 int *num_options, char *name, int type,
+					 int *num_options, const char *name, int type,
 					 dotconf_callback_t callback,
 					 info_t * info, unsigned long context)
 {
@@ -978,7 +989,7 @@ int module_audio_init(char **status_info)
 	gchar **outputs;
 	int i = 0;
 
-	DBG("Openning audio output system");
+	DBG("Opening audio output system");
 	if (NULL == module_audio_pars[0]) {
 		*status_info =
 		    g_strdup
@@ -987,6 +998,9 @@ int module_audio_init(char **status_info)
 		return -1;
 	}
 
+	g_free(module_audio_pars[6]);
+	module_audio_pars[6] = strdup(module_name);
+
 	outputs = g_strsplit(module_audio_pars[0], ",", 0);
 	while (NULL != outputs[i]) {
 		module_audio_id =
@@ -1051,7 +1065,7 @@ int module_play_file(const char *filename)
 
 	subformat = sfinfo.format & SF_FORMAT_SUBMASK;
 	items = sfinfo.channels * sfinfo.frames;
-	DBG("Frames = %ld, channels = %ld", sfinfo.frames,
+	DBG("Frames = %jd, channels = %ld", sfinfo.frames,
 	    (long)sfinfo.channels);
 	DBG("Samplerate = %i, items = %Ld", sfinfo.samplerate,
 	    (long long)items);
@@ -1089,3 +1103,112 @@ cleanup1:
 	sf_close(sf);
 	return result;
 }
+int module_marks_init(SPDMarks *marks)
+{
+	marks->num = 0;
+	marks->allocated = 0;
+	marks->samples = NULL;
+	marks->names = NULL;
+	marks->stop = 0;
+
+	return 0;
+}
+
+int module_marks_add(SPDMarks *marks, unsigned sample, const char *name)
+{
+	marks->num++;
+	if (marks->num >= marks->allocated) {
+		/* Amortized reallocation */
+		marks->allocated = marks->num * 2;
+		marks->samples = g_realloc(marks->samples, marks->allocated * sizeof(marks->samples[0]));
+		marks->names = g_realloc(marks->names, marks->allocated * sizeof(marks->names[0]));
+	}
+	marks->samples[marks->num - 1] = sample;
+	marks->names[marks->num - 1] = g_strdup(name);
+
+	return 0;
+}
+
+int module_marks_clear(SPDMarks *marks)
+{
+	unsigned i;
+
+	for (i = 0; i < marks->num; i++)
+		g_free(marks->names[i]);
+
+	marks->num = 0;
+	marks->allocated = 0;
+	g_free(marks->samples);
+	marks->samples = NULL;
+	g_free(marks->names);
+	marks->names = NULL;
+	marks->stop = 0;
+
+	return 0;
+}
+
+int module_tts_output_marks(AudioTrack track, AudioFormat format, SPDMarks *marks)
+{
+	AudioTrack cur = track;
+	int current_sample = 0;
+
+	/* Loop control */
+	int start = 0;
+	int end = marks->num;
+	int delta = 0; /* loop below figures out the delta */
+	int i;
+	for (i = 1; i < marks->num; i++) {
+		if (marks->samples[i - 1] > marks->samples[i]) {
+			/* Decreasing mark order */
+			if (delta > 0) {
+				DBG("WARNING: Mixed samples order");
+			} else {
+				start = marks->num - 1;
+				end = -1;
+				delta = -1;
+			}
+		} else if (marks->samples[i - 1] < marks->samples[i]) {
+			/* Increasing mark order */
+			if (delta < 0) {
+				DBG("WARNING: Mixed samples order");
+			} else {
+				delta = 1;
+			}
+		}
+	}
+	if (delta == 0) {
+		/* All marks are at the same sample */
+		delta = 1;
+	}
+
+	/* Alternate speaking and reporting mark */
+	for (i = start; i != end; i += delta) {
+		unsigned end_sample = marks->samples[i];
+
+		cur.samples = &track.samples[current_sample];
+		cur.num_samples = end_sample - current_sample;
+		current_sample = end_sample;
+
+		if (cur.num_samples && module_tts_output(cur, format))
+			return -1;
+		if (marks->stop)
+			return 1;
+		module_report_index_mark(marks->names[i]);
+	}
+
+	/* Finish with remaining bits if any */
+	if (track.num_samples > current_sample) {
+		cur.samples = &track.samples[current_sample];
+		cur.num_samples = track.num_samples - current_sample;
+		if (module_tts_output(cur, format))
+			return -1;
+	}
+
+	return 0;
+}
+
+int module_marks_stop(SPDMarks *marks)
+{
+	marks->stop = 1;
+	return 0;
+}
