@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: module_utils.h,v 1.22 2008-06-27 12:29:21 hanke Exp $
  */
@@ -43,6 +41,14 @@
 #include <speechd_types.h>
 #include "spd_audio.h"
 
+typedef struct SPDMarks {
+	unsigned num;
+	unsigned allocated;
+	unsigned *samples;
+	char **names;
+	gboolean stop;
+} SPDMarks;
+
 int log_level;
 
 AudioID *module_audio_id;
@@ -59,21 +65,27 @@ configfile_t *configfile;
 configoption_t *module_dc_options;
 int module_num_dc_options;
 
-#define CLEAN_OLD_SETTINGS_TABLE()\
+const char *module_name;
+
+#define CLEAN_OLD_SETTINGS_TABLE() do { \
 	msg_settings_old.rate = -101;\
 	msg_settings_old.pitch = -101;\
+	msg_settings_old.pitch_range = -101;\
 	msg_settings_old.volume = -101;\
-	msg_settings_old.punctuation_mode = -1;\
-	msg_settings_old.spelling_mode = -1;\
-	msg_settings_old.cap_let_recogn = -1;\
-	msg_settings_old.voice_type = -1;\
+	msg_settings_old.punctuation_mode = (SPDPunctuation) -1;\
+	msg_settings_old.spelling_mode = (SPDSpelling) -1;\
+	msg_settings_old.cap_let_recogn = (SPDCapitalLetters) -1;\
+	msg_settings_old.voice_type = (SPDVoiceType) -1;\
 	msg_settings_old.voice.name = NULL;\
-	msg_settings_old.voice.language = NULL;
+	msg_settings_old.voice.language = NULL; \
+} while (0)
 
-#define INIT_SETTINGS_TABLES()\
+#define INIT_SETTINGS_TABLES() do { \
+	module_name = MODULE_NAME; \
 	module_dc_options = NULL;\
 	msg_settings.rate = 0;\
 	msg_settings.pitch = 0;\
+	msg_settings.pitch_range = 0;\
 	msg_settings.volume = 0;\
 	msg_settings.punctuation_mode = SPD_PUNCT_NONE;\
 	msg_settings.spelling_mode = SPD_SPELL_OFF;\
@@ -81,9 +93,10 @@ int module_num_dc_options;
 	msg_settings.voice_type = SPD_MALE1;\
 	msg_settings.voice.name = NULL;\
 	msg_settings.voice.language = NULL;\
-	CLEAN_OLD_SETTINGS_TABLE()
+	CLEAN_OLD_SETTINGS_TABLE(); \
+} while (0)
 
-#define DBG(arg...) \
+#define DBG(arg...) do { \
 	if (Debug){ \
 		time_t t; \
 		struct timeval tv; \
@@ -105,16 +118,17 @@ int module_num_dc_options;
 			fflush(CustomDebugFile);			\
 		} \
 		g_free(tstr); \
-	}
+	} \
+} while(0)
 
-#define FATAL(msg) { \
+#define FATAL(msg) do { \
 		fprintf(stderr, "FATAL ERROR in output module [%s:%d]:\n   "msg, \
 		        __FILE__, __LINE__); \
 		if (Debug > 1) \
 			fprintf(CustomDebugFile, "FATAL ERROR in output module [%s:%d]:\n   "msg,	\
 			        __FILE__, __LINE__); \
 		exit(EXIT_FAILURE); \
-	}
+} while (0)
 
 int module_load(void);
 int module_init(char **status_info);
@@ -124,18 +138,25 @@ int module_stop(void);
 SPDVoice **module_get_voices(void);
 int module_tts_output(AudioTrack track, AudioFormat format);
 int module_play_file(const char *filename);
+int module_marks_init(SPDMarks *marks);
+int module_marks_add(SPDMarks *marks, unsigned sample, const char *name);
+int module_tts_output_marks(AudioTrack track, AudioFormat format, SPDMarks *marks);
+int module_marks_stop(SPDMarks *marks);
+int module_marks_clear(SPDMarks *marks);
 size_t module_pause(void);
 char *module_is_speaking(void);
 int module_close(void);
+SPDVoice **module_list_registered_voices(void);
 
-#define UPDATE_PARAMETER(value, setter) \
+#define UPDATE_PARAMETER(value, setter) do { \
 	if (msg_settings_old.value != msg_settings.value) \
 	{ \
 		msg_settings_old.value = msg_settings.value; \
 		setter (msg_settings.value); \
-	}
+	} \
+} while (0)
 
-#define UPDATE_STRING_PARAMETER(value, setter) \
+#define UPDATE_STRING_PARAMETER(value, setter) do { \
 	if (msg_settings_old.value == NULL || msg_settings.value == NULL \
 	        || strcmp (msg_settings_old.value, msg_settings.value)) \
 	{ \
@@ -149,7 +170,8 @@ int module_close(void);
 			msg_settings_old.value = g_strdup (msg_settings.value); \
 			setter (msg_settings.value); \
 		} \
-	}
+	} \
+} while (0)
 
 #define CHILD_SAMPLE_BUF_SIZE 16384
 
@@ -212,12 +234,12 @@ char *module_recode_to_iso(char *data, int bytes, char *language,
 			   char *fallback);
 void module_signal_end(void);
 configoption_t *module_add_config_option(configoption_t * options,
-					 int *num_options, char *name, int type,
+					 int *num_options, const char *name, int type,
 					 dotconf_callback_t callback,
 					 info_t * info, unsigned long context);
 
 configoption_t *add_config_option(configoption_t * options,
-				  int *num_config_options, char *name, int type,
+				  int *num_config_options, const char *name, int type,
 				  dotconf_callback_t callback, info_t * info,
 				  unsigned long context);
 
@@ -227,9 +249,10 @@ configoption_t *add_config_option(configoption_t * options,
 	static char *name = NULL; \
 	DOTCONF_CB(name ## _cb) \
 	{ \
-		g_free(name); \
-		if (cmd->data.str != NULL) \
+		if (cmd->data.str != NULL) { \
+			g_free(name); \
 			name = g_strdup(cmd->data.str); \
+		} \
 		return NULL; \
 	}
 
@@ -318,37 +341,42 @@ configoption_t *add_config_option(configoption_t * options,
 		return NULL; \
 	}
 
-#define MOD_OPTION_1_STR_REG(name, default) \
+#define MOD_OPTION_1_STR_REG(name, default) do { \
 	if (default != NULL) name = g_strdup(default); \
 	else name = NULL; \
 	module_dc_options = module_add_config_option(module_dc_options, \
 	                    &module_num_dc_options, #name, \
-	                    ARG_STR, name ## _cb, NULL, 0);
+	                    ARG_STR, name ## _cb, NULL, 0); \
+} while (0)
 
-#define MOD_OPTION_1_INT_REG(name, default) \
+#define MOD_OPTION_1_INT_REG(name, default) do { \
 	name = default; \
 	module_dc_options = module_add_config_option(module_dc_options, \
 	                    &module_num_dc_options, #name, \
-	                    ARG_INT, name ## _cb, NULL, 0);
+	                    ARG_INT, name ## _cb, NULL, 0); \
+} while (0)
 
 /* TODO: Switch this to real float, not /100 integer,
    as soon as DotConf supports floats */
-#define MOD_OPTION_1_FLOAT_REG(name, default) \
+#define MOD_OPTION_1_FLOAT_REG(name, default) do { \
 	name = default; \
 	module_dc_options = module_add_config_option(module_dc_options, \
 	                    &module_num_dc_options, #name, \
-	                    ARG_INT, name ## _cb, NULL, 0);
+	                    ARG_INT, name ## _cb, NULL, 0); \
+} while (0)
 
-#define MOD_OPTION_MORE_REG(name) \
+#define MOD_OPTION_MORE_REG(name) do { \
 	module_dc_options = module_add_config_option(module_dc_options, \
 	                    &module_num_dc_options, #name, \
-	                    ARG_LIST, name ## _cb, NULL, 0);
+	                    ARG_LIST, name ## _cb, NULL, 0); \
+} while (0)
 
-#define MOD_OPTION_HT_REG(name) \
+#define MOD_OPTION_HT_REG(name) do { \
 	name = g_hash_table_new(g_str_hash, g_str_equal); \
 	module_dc_options = module_add_config_option(module_dc_options, \
 	                    &module_num_dc_options, #name, \
-	                    ARG_LIST, name ## _cb, NULL, 0);
+	                    ARG_LIST, name ## _cb, NULL, 0); \
+} while (0)
 
 /* --- DEBUGGING SUPPORT  ---*/
 
@@ -386,7 +414,9 @@ int module_utils_init(void);
 int module_audio_init(char **status_info);
 
 	/* Prototypes from module_utils_addvoice.c */
+void module_register_available_voices(void);
 void module_register_settings_voices(void);
 char *module_getvoice(char *language, SPDVoiceType voice);
+gboolean module_existsvoice(char *voicename);
 
 #endif /* #ifndef __MODULE_UTILS_H */
