@@ -15,18 +15,16 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: spd_audio.c,v 1.21 2008-06-09 10:29:12 hanke Exp $
  */
 
-/* 
+/*
  * spd_audio is a simple realtime audio output library with the capability of
  * playing 8 or 16 bit data, immediate stop and synchronization. This library
  * currently provides OSS, NAS, ALSA and PulseAudio backend. The available backends are
- * specified at compile-time using the directives WITH_OSS, WITH_NAS, WITH_ALSA, 
+ * specified at compile-time using the directives WITH_OSS, WITH_NAS, WITH_ALSA,
  * WITH_PULSE, WITH_LIBAO but the user program is allowed to switch between them at run-time.
  */
 
@@ -38,7 +36,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <time.h>
