@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: compare.h,v 1.3 2006-07-11 16:12:27 hanke Exp $
  */
@@ -25,11 +23,10 @@
 #ifndef COMPARE_H
 #define COMPARE_H
 
-gint compare_message_fd(gconstpointer element, gconstpointer value, gpointer x);
 gint compare_message_uid(gconstpointer element, gconstpointer value,
 			 gpointer x);
 
-/* Pointers to functions compare_message_fd and compare_message_uid */
+/* Pointer to function compare_message_uid */
 gint(*p_msg_lc) ();
 gint(*p_msg_uid_lc) ();
 
