@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: parse.c,v 1.73 2008-07-01 08:52:31 hanke Exp $
  */
@@ -458,9 +456,39 @@ char *parse_set(const char *buf, const int bytes, const int fd,
 			return g_strdup(ERR_COULDNT_SET_LANGUAGE);
 		return g_strdup(OK_LANGUAGE_SET);
 	} else if (TEST_CMD(set_sub, "synthesis_voice")) {
-		char *synthesis_voice;
+		char *synthesis_voice = NULL;
+		char *tmp = NULL;
+		gchar **split_command;
+		int i;
 
-		GET_PARAM_STR(synthesis_voice, 3, 0);
+		split_command = g_strsplit(buf, " ", 0);
+		for (i = 0; split_command[i] != NULL; i++) {
+			tmp = NULL;
+
+			if (i == 3) {
+				if (g_str_has_suffix(split_command[i], "\n")) {
+					synthesis_voice = g_strndup(split_command[i], strlen(split_command[i]) - 2);
+				} else {
+					synthesis_voice = g_strdup(split_command[i]);
+				}
+			} else if (i > 3) {
+				if (g_str_has_suffix(split_command[i], "\n")) {
+					tmp = g_strndup(split_command[i], strlen(split_command[i]) - 2);
+					g_free(split_command[i]);
+					split_command[i] = tmp;
+					tmp = NULL;
+				}
+
+				tmp = g_strjoin(" ", synthesis_voice, split_command[i], NULL);
+				g_free(synthesis_voice);
+				synthesis_voice = tmp;
+			}
+		}
+
+		g_strfreev(split_command);
+
+		if (i < 3)
+			synthesis_voice = NULL;
 
 		SSIP_SET_COMMAND(synthesis_voice);
 		g_free(synthesis_voice);
@@ -510,6 +538,19 @@ char *parse_set(const char *buf, const int bytes, const int fd,
 		if (ret)
 			return g_strdup(ERR_COULDNT_SET_PITCH);
 		return g_strdup(OK_PITCH_SET);
+	} else if (TEST_CMD(set_sub, "pitch_range")) {
+		signed int pitch_range;
+		GET_PARAM_INT(pitch_range, 3);
+
+		if (pitch_range < -100)
+			return g_strdup(ERR_PITCH_RANGE_TOO_LOW);
+		if (pitch_range > +100)
+			return g_strdup(ERR_PITCH_RANGE_TOO_HIGH);
+
+		SSIP_SET_COMMAND(pitch_range);
+		if (ret)
+			return g_strdup(ERR_COULDNT_SET_PITCH_RANGE);
+		return g_strdup(OK_PITCH_RANGE_SET);
 	} else if (TEST_CMD(set_sub, "volume")) {
 		signed int volume;
 		GET_PARAM_INT(volume, 3);
@@ -523,16 +564,13 @@ char *parse_set(const char *buf, const int bytes, const int fd,
 		if (ret)
 			return g_strdup(ERR_COULDNT_SET_VOLUME);
 		return g_strdup(OK_VOLUME_SET);
-	} else if (TEST_CMD(set_sub, "voice") || TEST_CMD(set_sub, "voice_type")) {
+	} else if (TEST_CMD(set_sub, "voice_type")) {
 		char *voice;
 		GET_PARAM_STR(voice, 3, CONV_DOWN);
 
 		SSIP_SET_COMMAND(voice);
 		g_free(voice);
 
-		if (TEST_CMD(set_sub, "voice"))
-			MSG(1, "The SET VOICE command is deprecated since Speech Dispatcher 0.8.3.");
-
 		if (ret)
 			return g_strdup(ERR_COULDNT_SET_VOICE);
 		return g_strdup(OK_VOICE_SET);
@@ -802,9 +840,6 @@ char *parse_general_event(const char *buf, const int bytes, const int fd,
 
 	GET_PARAM_STR(param, 1, NO_CONV);
 
-	if (param == NULL)
-		return g_strdup(ERR_MISSING_PARAMETER);
-
 	if (param[0] == 0) {
 		g_free(param);
 		return g_strdup(ERR_MISSING_PARAMETER);
@@ -884,8 +919,11 @@ char *parse_list(const char *buf, const int bytes, const int fd,
 		len = g_list_length(output_modules);
 		for (i = 0; i < len; i++) {
 			mod = g_list_nth_data(output_modules, i);
-			g_string_append_printf(result, C_OK_MODULES "-%s" NEWLINE,
-					       mod->name);
+			if (strcmp(mod->name, "dummy") &&
+			    strcmp(mod->name, "generic"))
+				g_string_append_printf(result, C_OK_MODULES
+						       "-%s" NEWLINE,
+						       mod->name);
 		}
 
 		g_string_append(result, OK_MODULES_LIST_SENT);
@@ -918,7 +956,7 @@ char *parse_list(const char *buf, const int bytes, const int fd,
 			if (voices[i] == NULL)
 				break;
 			g_string_append_printf(result,
-					       C_OK_VOICES "-%s %s %s" NEWLINE,
+					       C_OK_VOICES "-%s\t%s\t%s" NEWLINE,
 					       voices[i]->name,
 					       voices[i]->language,
 					       voices[i]->variant);
