@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: speechd.c,v 1.81 2008-07-10 15:36:49 hanke Exp $
  */
@@ -32,6 +30,10 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
+#ifdef HAVE_SYS_FILIO_H
+#include <sys/filio.h>	/* Needed for FIONREAD on Solaris */
+#endif
+
 #include "speechd.h"
 
 /* Declare dotconf functions and data structures*/
@@ -78,10 +80,9 @@ static gboolean client_process_incoming (gint          fd,
 
 void check_client_count(void);
 
-#ifdef __SUNPRO_C
-/* Added by Willie Walker - daemon is a gcc-ism
+#ifndef HAVE_DAEMON
+/* Added by Willie Walker - daemon is a common, but not universal, extension.
  */
-#include <sys/filio.h>
 static int daemon(int nochdir, int noclose)
 {
 	int fd, i;
@@ -115,7 +116,7 @@ static int daemon(int nochdir, int noclose)
 	}
 	return 0;
 }
-#endif /* __SUNPRO_C */
+#endif /* HAVE_DAEMON */
 
 /* --- DEBUGGING --- */
 
@@ -696,8 +697,9 @@ static gboolean speechd_load_configuration(gpointer user_data)
 
 		/* We need to load modules here, since this is called both by speechd_init
 		 * and to handle SIGHUP. */
-		if (module_number_of_requested_modules() < 2) {
-			detected_modules = detect_output_modules(SpeechdOptions.module_dir);
+		if (module_number_of_requested_modules() < 1) {
+			detected_modules = detect_output_modules(SpeechdOptions.module_dir,
+								 SpeechdOptions.conf_dir);
 			while (detected_modules != NULL) {
 				char **parameters = detected_modules->data;
 				module_add_load_request(parameters[0],
@@ -967,13 +969,6 @@ int main(int argc, char *argv[])
 	int spawn_port = 0;
 	char *spawn_socket_path = NULL;
 
-	/* Initialize threading and thread safety in Glib
-	 * Deprecated in 2.32
-	 */
-#ifndef GLIB_VERSION_2_32
-	g_thread_init(NULL);
-#endif
-
 	/* Strip all permisions for 'others' from the files created */
 	umask(007);
 
