@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: speechd.h,v 1.64 2008-06-27 12:29:07 hanke Exp $
  */
@@ -82,6 +80,7 @@ typedef struct {
 	int paused_while_speaking;
 	SPDMessageType type;	/* Type of the message (1=text, 2=icon, 3=char, 4=key) */
 	SPDDataMode ssml_mode;	/* SSML mode on (1)/off (0) */
+	gboolean symbols_preprocessing; /* whether to transform symbols/punctuation in the input */
 	SPDPriority priority;	/* Priority between 1 and 5 (1 - highest, 5 - lowest) */
 	SPDMsgSettings msg_settings;
 	char *client_name;	/* Name of the client. */
@@ -232,8 +231,8 @@ TSpeechDSock *speechd_socket_get_by_fd(int fd);
 /* Debugging */
 void MSG(int level, char *format, ...);
 void MSG2(int level, char *kind, char *format, ...);
-#define FATAL(msg) { fatal_error(); MSG(-1,"Fatal error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); }
-#define DIE(msg) { MSG(0,"Error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); }
+#define FATAL(msg) do { fatal_error(); MSG(-1,"Fatal error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); } while (0)
+#define DIE(msg) do { MSG(0,"Error [%s:%d]:"msg, __FILE__, __LINE__); exit(EXIT_FAILURE); } while (0)
 
 FILE *logfile;
 FILE *custom_logfile;
